from datetime import datetime

from okta.models.user.LoginCredentials import LoginCredentials
from okta.models.user.UserProfile import UserProfile
from okta.models.Link import Link


class User:

    types = {
        'id': str,
        'status': str,
        'created': datetime,
        'activated': datetime,
        'statusChanged': datetime,
        'lastLogin': datetime,
        'lastUpdated': datetime,
        'passwordChanged': datetime,
        'transitioningToStatus': str,
        'profile': UserProfile,
        'credentials': LoginCredentials
    }

    dict_types = {
        '_links': Link
    }

    alt_names = {
        '_links': 'links'
    }

    def __init__(self, **kwargs):

        # unique key for user
        self.id = None  # str

        # current status of user
        self.status = None  # str

        # timestamp when user was created
        self.created = None  # datetime

        # timestamp when transition to ACTIVE status completed
        self.activated = None  # datetime

        # timestamp when status last changed
        self.statusChanged = None  # datetime

        # timestamp of last login
        self.lastLogin = None  # datetime

        # timestamp when user was last updated
        self.lastUpdated = None  # datetime

        # timestamp when password last changed
        self.passwordChanged = None  # datetime

        # target status of an in progress asynchronous status transition
        self.transitioningToStatus = None  # str

        self.profile = None  # UserProfile

        self.credentials = None  # LoginCredentials

        self.links = None

        # Populate profile
        profile_attrs = ['login', 'email', 'secondEmail', 'firstName', 'lastName', 'mobilePhone']
        for attr in profile_attrs:
            if attr in kwargs:
                self.profile = self.profile or UserProfile()
                setattr(self.profile, attr, kwargs[attr])


"""
    okta
    ~~~~~
    An SDK using the Okta APIs to manage Okta instances.

    :copyright: (c) 2015 by Okta.
    :license: Apache 2, see LICENSE.txt for more details.
"""

__version__ = '0.0.3a'

from .AppInstanceClient import AppInstanceClient
from .AuthClient import AuthClient
from .EventsClient import EventsClient
from .FactorsAdminClient import FactorsAdminClient
from .FactorsClient import FactorsClient
from .SessionsClient import SessionsClient
from .UserGroupsClient import UserGroupsClient
from .UsersClient import UsersClient



class UserNameTemplate:

    types = {
        'template': str,
        'type': str,
        'userSuffix': str
    }

    def __init__(self):

        # mapping expression for username
        self.template = None  # str

        # type of mapping expression
        self.type = None  # enum

        # suffix for built-in mapping expressions
        self.userSuffix = None  # str



from datetime import datetime
from okta.models.app.Accessibility import Accessibility
from okta.models.app.Visibility import Visibility
from okta.models.app.AppCredentials import AppCredentials
from okta.models.app.Settings import Settings
from okta.models.app.AppSettings import AppSettings


class AppInstance:

    types = {
        'id': str,
        'name': str,
        'label': str,
        'created': datetime,
        'lastUpdated': datetime,
        'status': str,
        'activated': datetime,
        'features': str,
        'signOnMode': str,
        'accessibility': Accessibility,
        'visibility': Visibility,
        'credentials': AppCredentials,
        'settings': Settings
    }

    def __init__(self):

        # unique key for app
        self.id = None  # str

        # unique key for app definition
        self.name = None  # str

        # unique user-defined display name for app
        self.label = None  # str

        # timestamp when app was created
        self.created = None  # datetime

        # timestamp when app was last updated
        self.lastUpdated = None  # datetime

        # status of app
        self.status = None  # enum

        # timestamp when transition to ACTIVE status completed
        self.activated = None  # datetime

        # enabled app features
        self.features = None  # enum

        # authentication mode of app
        self.signOnMode = None  # enum

        self.accessibility = None  # Accessibility

        self.visibility = None  # Visibility

        self.credentials = None  # AppCredentials

        self.settings = None  # Settings

    @staticmethod
    def build_bookmark(url, label=None, request_integration=False):
        app = AppInstance()
        app.name = "bookmark"
        app.label = label
        app.signOnMode = "BOOKMARK"
        app.settings = Settings()
        app.settings.app = AppSettings()
        app.settings.app.url = url
        app.settings.app.requestIntegration = request_integration
        return app


from okta.framework.ApiClient import ApiClient
from okta.framework.Utils import Utils
from okta.models.auth.AuthResult import AuthResult


class AuthClient(ApiClient):
    def __init__(self, base_url, api_token):
        ApiClient.__init__(self, base_url + '/api/v1/authn', api_token)

    def authenticate(self, username, password,
                     relay_state=None, response_type=None, force_mfa=None, context=None):
        """Begin the authentication process with a username and password

        :param username: user's username
        :type username: str
        :param password: user's password
        :type password: str
        :param relay_state: data that will persist for the lifetime of the authentication or recovery token
        :type relay_state: str or None
        :param response_type: the type of session to return (session_token or session_token_url usually)
        :type response_type: str
        :param force_mfa: whether to force mfa even if the auth is exempt
        :type force_mfa: bool
        :param context: contextual info about the auth request like ip and location
        :type context: Context
        :rtype: AuthResult
        """
        request = {
            'username': username,
            'password': password,
            'relayState': relay_state,
            'context': context
        }

        params = {
            'force_mfa': force_mfa,
            'response_type': response_type
        }

        response = ApiClient.post_path(self, '/', request, params=params)
        return Utils.deserialize(response.text, AuthResult)

    def auth_with_factor(self, state_token, factor_id, passcode,
                         relay_state=None, remember_device=None):
        """Continue authentication with an MFA attempt

        :param state_token: current state token from the previous AuthResult
        :type state_token: str
        :param factor_id: target factor id
        :type factor_id: str
        :param passcode: passcode required for authenticating the factor
        :type passcode: str
        :param relay_state: data that will persist for the lifetime of the authentication or recovery token
        :type relay_state: str or None
        :param remember_device: whether to remember this device to avoid requiring MFA next time
        :type remember_device: bool
        :rtype: AuthResult
        """
        request = {
            'stateToken': state_token,
            'passCode': passcode,
            'relayState': relay_state
        }

        params = {
            'rememberDevice': remember_device
        }

        response = ApiClient.post_path(self, '/factors/{0}/verify'.format(factor_id),
                                      request, params=params)
        return Utils.deserialize(response.text, AuthResult)

    # MFA MANAGEMENT

    def enroll_factor(self, state_token, factor_type, provider, profile, relay_state=None):
        """Enroll in an MFA factor during the auth flow. Usually only encountered if MFA is required for authentication

        :param state_token: current state token from the previous AuthResult
        :type state_token: str
        :param factor_type: type of factor (sms, token, question, token:software:totp, token:hardware etc)
        :type factor_type: str
        :param provider: factor provider (OKTA, RSA, SYMANTEC, GOOGLE etc)
        :type provider: str
        :param profile: factor profile that depends on the factor type
        :type profile: FactorProfile
        :param relay_state: data that will persist for the lifetime of the authentication or recovery token
        :type relay_state: str or None
        :rtype: AuthResult
        """
        request = {
            'stateToken': state_token,
            'factorType': factor_type,
            'provider': provider,
            'profile': profile,
            'relayState': relay_state
        }

        response = ApiClient.post_path(self, '/factors', request)
        return Utils.deserialize(response.text, AuthResult)

    def activate_factor(self, state_token, factor_id, passcode, relay_state=None):
        """Activate an MFA factor during the auth flow

        :param state_token: current state token from the previous AuthResult
        :type state_token: str
        :param factor_id: target factor id
        :type factor_id: str
        :param passcode: passcode required to activate the factor
        :type passcode: str
        :param relay_state: data that will persist for the lifetime of the authentication or recovery token
        :type relay_state: str or None
        :rtype: AuthResult
        """
        request = {
            'stateToken': state_token,
            'passCode': passcode,
            'relayState': relay_state
        }

        response = ApiClient.post_path(self, '/factors/{0}/lifecycle/activate'.format(factor_id), request)
        return Utils.deserialize(response.text, AuthResult)

    def resend_code(self, state_token, factor_id, relay_state=None):
        """Resend an a passcode for an authentication factor

        :param state_token: current state token from the previous AuthResult
        :type state_token: str
        :param factor_id: target factor id
        :type factor_id: str
        :param relay_state: data that will persist for the lifetime of the authentication or recovery token
        :type relay_state: str or None
        :rtype: AuthResult
        """
        request = {
            'stateToken': state_token,
            'relayState': relay_state
        }

        response = ApiClient.post_path(self, '/factors/{0}/lifecycle/resend'.format(factor_id), request)
        return Utils.deserialize(response.text, AuthResult)

    # CREDENTIAL MANAGEMENT

    def change_password(self, state_token, old_password, new_password, relay_state=None):
        """Change a user's password during an authentication flow

        :param state_token: current state token from the previous AuthResult
        :type state_token: str
        :param old_password: user's current password
        :type old_password: str
        :param new_password: user's desired password
        :type new_password: str
        :param relay_state: data that will persist for the lifetime of the authentication or recovery token
        :type relay_state: str or None
        :rtype: AuthResult
        """
        request = {
            'stateToken': state_token,
            'oldPassword': old_password,
            'newPassword': new_password,
            'relayState': relay_state
        }

        response = ApiClient.post_path(self, '/credentials/change_password', request)
        return Utils.deserialize(response.text, AuthResult)

    def reset_password(self, state_token, new_password, relay_state=None):
        """Reset a user's password during an authentication flow

        :param state_token: current state token from the previous AuthResult
        :type state_token: str
        :param new_password: user's desired password
        :type new_password: str
        :param relay_state: data that will persist for the lifetime of the authentication or recovery token
        :type relay_state: str or None
        :rtype: AuthResult
        """
        request = {
            'stateToken': state_token,
            'newPassword': new_password,
            'relayState': relay_state
        }

        response = ApiClient.post_path(self, '/credentials/reset_password', request)
        return Utils.deserialize(response.text, AuthResult)

    def forgot_password(self, username, relay_state=None):
        """Initiate a forgot password flow for a user

        :param username: user's username
        :type username: str
        :param relay_state: data that will persist for the lifetime of the authentication or recovery token
        :type relay_state: str or None
        :rtype: AuthResult
        """
        request = {
            'username': username,
            'relayState': relay_state
        }

        response = ApiClient.post_path(self, '/recovery/password', request)
        return Utils.deserialize(response.text, AuthResult)

    def forgot_password_answer(self, state_token, security_answer, new_password, relay_state=None):
        """Answer the forgot password during an authentication flow

        :param state_token: current state token from the previous AuthResult
        :type state_token: str
        :param security_answer: answer to a user's security question
        :type security_answer: str
        :param new_password: user's desired password
        :type new_password: str
        :param relay_state: data that will persist for the lifetime of the authentication or recovery token
        :type relay_state: str or None
        :rtype: AuthResult
        """
        request = {
            'stateToken': state_token,
            'securityAnswer': security_answer,
            'newPassword': new_password,
            'relayState': relay_state
        }

        response = ApiClient.post_path(self, '/recovery/answer', request)
        return Utils.deserialize(response.text, AuthResult)

    # RECOVERY

    def validate_recovery_token(self, recovery_token, relay_state=None):
        """Validate a token for recovery

        :param recovery_token: token distributed to end-user via out-of-band mechanism such as email
        :type recovery_token: str
        :param relay_state: data that will persist for the lifetime of the authentication or recovery token
        :type relay_state: str or None
        :rtype: AuthResult
        """
        request = {
            'recoveryToken': recovery_token,
            'relayState': relay_state
        }

        response = ApiClient.post_path(self, '/recovery/token', request)
        return Utils.deserialize(response.text, AuthResult)

    def unlock_account(self, username, relay_state=None):
        """Begin unlocking an account

        :param username: user's username
        :type username: str
        :param relay_state: data that will persist for the lifetime of the authentication or recovery token
        :type relay_state: str or None
        :rtype: AuthResult
        """
        request = {
            'username': username,
            'relayState': relay_state
        }

        response = ApiClient.post_path(self, '/recovery/unlock', request)
        return Utils.deserialize(response.text, AuthResult)

    def unlock_account_answer(self, state_token, security_answer, relay_state=None):
        """Unlock an account during an authentication

        :param state_token: current state token from the previous AuthResult
        :type state_token: str
        :param security_answer: answer to the user's security question
        :type security_answer: str
        :param relay_state: data that will persist for the lifetime of the authentication or recovery token
        :type relay_state: str or None
        :rtype: AuthResult
        """
        request = {
            'stateToken': state_token,
            'securityAnswer': security_answer,
            'relayState': relay_state
        }

        response = ApiClient.post_path(self, '/recovery/answer', request)
        return Utils.deserialize(response.text, AuthResult)

    # STATE MANAGEMENT

    def previous_state(self, state_token, relay_state=None):
        """Get the previous state of an in-progress authentication

        :param state_token: current state token from the previous AuthResult
        :type state_token: str
        :param relay_state: data that will persist for the lifetime of the authentication or recovery token
        :type relay_state: str or None
        :rtype: AuthResult
        """
        request = {
            'stateToken': state_token,
            'relayState': relay_state
        }

        response = ApiClient.post_path(self, '/previous', request)
        return Utils.deserialize(response.text, AuthResult)

    def get_status(self, state_token, relay_state=None):
        """Get the status of an in-progress authentication

        :param state_token: current state token from the previous AuthResult
        :type state_token: str
        :param relay_state: data that will persist for the lifetime of the authentication or recovery token
        :type relay_state: str or None
        :rtype: AuthResult
        """
        request = {
            'stateToken': state_token,
            'relayState': relay_state
        }

        response = ApiClient.post_path(self, '/', request)
        return Utils.deserialize(response.text, AuthResult)

    def verify_transaction(self, factor_id, transaction_id, user_response):
        """Verify a transaction

        :param factor_id: target factor id
        :type factor_id: str
        :param transaction_id: target transaction id
        :type transaction_id: str
        :param user_response: APPROVE or REJECT
        :type user_response: str
        :rtype: AuthResult
        """
        request = {
            'result': user_response
        }

        response = ApiClient.post_path(self, '/factors/{0}/transactions/{1}/verify'.format(factor_id, transaction_id), request)
        return Utils.deserialize(response.text, AuthResult)


from okta.models.user import Password, RecoveryQuestion


class ChangeRecoveryQuestionRequest:

    types = {
        'password': Password,
        'recovery_question': RecoveryQuestion
    }

    def __init__(self):

        self.password = None  # Password

        self.recovery_question = None  # RecoveryQuestion



class OrgAuthFactor:

    types = {
        'id': str,
        'factorType': str,
        'provider': str,
        'status': str
    }

    def __init__(self):

        self.id = None  # str

        self.factorType = None  # str

        self.provider = None  # str

        self.status = None  # str

        self.links = None  # Map<String, Object>

        self.embedded = None  # Map<String, Object>



class Action:

    types = {
        'message': str,
        'categories': str,
        'objectType': str,
        'requestUri': str
    }

    def __init__(self):

        # Description of an action
        self.message = None  # str

        # Categories for an action
        self.categories = None

        # Identifies the unique type of an action
        self.objectType = None  # str

        # Relative uri of the request that generated the event.
        self.requestUri = None  # str



from okta.models.app.UserNameTemplate import UserNameTemplate
from okta.models.user.Password import Password


class AppCredentials:

    types = {
        'scheme': str,
        'userNameTemplate': UserNameTemplate,
        'userName': str,
        'password': Password
    }

    def __init__(self):

        # Determines how credentials are managed for the signOnMode
        self.scheme = None  # enum

        # Template used to generate a users username when the application is assigned via a group or directly to a user
        self.userNameTemplate = None  # UserNameTemplate

        # Shared username for app
        self.userName = None  # str

        # Shared password for app
        self.password = None  # Password



from okta.framework.ApiClient import ApiClient
from okta.framework.Utils import Utils
from okta.models.factor.OrgAuthFactor import OrgAuthFactor


class FactorsAdminClient(ApiClient):
    def __init__(self, base_url, api_token):
        ApiClient.__init__(self, base_url + '/api/v1/org', api_token)

    def get_org_factors(self, filter_string=None):
        """Get a list of OrgAuthFactors

        :param filter_string: string to filter factors
        :type filter_string: str or None
        :rtype: list of OrgAuthFactor
        """
        params = {
            'filter': filter_string
        }
        response = ApiClient.get_path(self, '/factors', params=params)
        return Utils.deserialize(response.text, OrgAuthFactor)

    def activate_org_factor(self, org_factor_id, org_auth_factor=None):
        """Activate OrgAuthFactor

        :param org_factor_id: target factor id
        :type org_factor_id: str
        :param org_auth_factor: additional factor data
        :param org_auth_factor: OrgAuthFactor
        :rtype: OrgAuthFactor
        """
        response = ApiClient.post_path(self, '/factors/{0}/lifecycle/activate'.format(org_factor_id), org_auth_factor)
        return Utils.deserialize(response.text, OrgAuthFactor)

    def deactivate_org_factor(self, org_factor_id):
        """Deactivate OrgAuthFactor

        :param org_factor_id: target factor id
        :type org_factor_id: str
        :rtype: OrgAuthFactor
        """
        response = ApiClient.post_path(self, '/factors/{0}/lifecycle/deactivate'.format(org_factor_id))
        return Utils.deserialize(response.text, OrgAuthFactor)


from okta.framework.Utils import Utils


class PagedResults(object):

    def __init__(self, response, target_class):
        self.response = response
        self.__target_class = target_class

    def is_last_page(self):
        return not ("next" in self.response.links)

    def is_first_page(self):
        return not ("previous" in self.response.links)

    def is_only_page(self):
        return self.is_last_page() and self.is_first_page()

    @property
    def next_url(self):
        return self.response.links.get("next").get("url")

    @property
    def result(self):
        return Utils.deserialize(self.response.text, self.__target_class)


from Verification import Verification


class FactorDeviceEnrollRequest:

    types = {
        'verify': Verification,
        'profile': dict
    }

    def __init__(self):

        self.verify = None  # Verification

        self.profile = None  # Map<String, Object>



import json
import dateutil.parser
from datetime import datetime
import types
import six
import inspect


class Utils(object):
    @staticmethod
    def deserialize(from_data, to_class):

        def custom_setattr(o, a, v):
            if hasattr(o, 'alt_names') and a in o.alt_names:
                a = o.alt_names[a]

            setattr(o, a, v)

        json_dump = {}
        if from_data is None or len(from_data) == 0:
            json_dump = {}
        elif isinstance(from_data, six.text_type) or isinstance(from_data, six.string_types):
            json_dump = json.loads(from_data)
        else:
            json_dump = from_data

        list_type = types.ListType if six.PY2 else list
        if isinstance(json_dump, list_type):
            obj_list = list()
            for obj in json_dump:
                obj_list.append(Utils.deserialize(obj, to_class))

            return obj_list

        else:
            obj = to_class()

            # Loop through each type of object in types
            if hasattr(to_class, 'types'):
                for attr, attr_type in six.iteritems(to_class.types):
                    if attr in json_dump:
                        val = json_dump[attr]

                        if not val:
                            continue

                        if attr_type == datetime:
                            val = dateutil.parser.parse(val)

                        elif attr_type == str or attr_type == int or attr_type == dict or attr_type == bool:
                            pass

                        else:
                            val = Utils.deserialize(val, attr_type)

                        custom_setattr(obj, attr, val)

            # Otherwise, assume all the properties are strings
            else:
                for key, value in six.iteritems(json_dump):
                    if key in to_class.__dict__:
                        custom_setattr(obj, key, json_dump[key])

            # Some models have dicts as values
            if hasattr(to_class, 'dict_types'):
                for attr, attr_type in six.iteritems(to_class.dict_types):
                    if attr in json_dump:
                        dictionary = json_dump[attr]
                        new_dict = dict()

                        for key, value in six.iteritems(dictionary):
                            val = Utils.deserialize(value, attr_type)
                            new_dict[key] = val

                        custom_setattr(obj, attr, new_dict)

            return obj

    @staticmethod
    def remove_nulls(d):
        built = {}
        for k, v in six.iteritems(d):
            if v is None:
                continue

            if isinstance(v, dict):
                built[k] = Utils.remove_nulls(v)

            if isinstance(v, object) and hasattr(v, '__dict__'):
                built[k] = Utils.remove_nulls(v.__dict__)

            else:
                built[k] = v

        return built


from datetime import datetime
from okta.models.event.Action import Action
from okta.models.event.Actor import Actor
from okta.models.event.Target import Target


class Event:

    types = {
        'eventId': str,
        'published': datetime,
        'requestId': str,
        'sessionId': str,
        'action': Action,
        'actors': Actor,
        'targets': Target
    }

    def __init__(self):

        # unique key for event
        self.id = None  # str

        # timestamp when event was published  
        self.published = None  # datetime

        # identifies the request
        self.requestId = None  # str

        # session in which the event occurred
        self.sessionId = None  # str

        self.action = None  # Action

        self.actors = None

        self.targets = None



class Credentials:

    types = {
        'username': str,
        'password': str
    }

    def __init__(self):

        # a user&#39;s login name
        self.username = None  # str

        # a user&#39;s password
        self.password = None  # str



class Hide:

    types = {
        'iOS': bool,
        'web': bool
    }

    def __init__(self):

        # Okta Mobile for iOS or Android (pre-dates Android)
        self.iOS = None  # bool

        # Okta Web Browser Home Page
        self.web = None  # bool



from okta.framework.ApiClient import ApiClient
from okta.framework.Utils import Utils
from okta.models.factor.FactorCatalogEntry import FactorCatalogEntry
from okta.models.factor.Factor import Factor
from okta.models.factor.Question import Question
from okta.models.factor.FactorVerificationResponse import FactorVerificationResponse
from okta.models.factor.FactorDevice import FactorDevice


class FactorsClient(ApiClient):

    def __init__(self, base_url, api_token):
        ApiClient.__init__(self, base_url + '/api/v1/users', api_token)

    def get_factors_catalog(self, user_id):
        """Get available factors for a user

        :param user_id: target user id
        :type user_id: str
        :rtype: list of FactorCatalogEntry
        """
        response = ApiClient.get_path(self, '/{0}/factors/catalog'.format(user_id))
        return Utils.deserialize(response.text, FactorCatalogEntry)

    def get_lifecycle_factors(self, user_id):
        """Get enrolled factors for a user

        :param user_id: target user id
        :type user_id: str
        :rtype: list of Factor
        """
        response = ApiClient.get_path(self, '/{0}/factors'.format(user_id))
        return Utils.deserialize(response.text, Factor)

    # FACTOR CRUD

    def get_available_questions(self, user_id):
        """Get available factor questions

        :param user_id: target user id
        :type user_id: str
        :rtype: list of Question
        """
        response = ApiClient.get_path(self, '/{0}/factors/questions'.format(user_id))
        return Utils.deserialize(response.text, Question)

    def enroll_factor(self, user_id, factor_enroll_request, update_phone=None):
        """Enroll a user into a factor

        :param user_id: target user id
        :type user_id: str
        :param factor_enroll_request: the details to enroll the user
        :type factor_enroll_request: FactorEnrollRequest
        :param update_phone: whether to update the user's phone during enrollment
        :type update_phone: bool
        :rtype: Factor
        """
        params = {
            'updatePhone': update_phone
        }
        response = ApiClient.post_path(self, '/{0}/factors'.format(user_id), factor_enroll_request, params=params)
        return Utils.deserialize(response.text, Factor)

    def get_factor(self, user_id, user_factor_id):
        """Get information about an enrolled factor

        :param user_id: target user id
        :type user_id: str
        :param user_factor_id: target factor id
        :type user_factor_id: str
        :rtype: Factor
        """
        response = ApiClient.get_path(self, '/{0}/factors/{1}'.format(user_id, user_factor_id))
        return Utils.deserialize(response.text, Factor)

    def update_factor(self, user_id, user_factor_id, factor_enroll_request):
        """Update an enrolled factor

        :param user_id: target user id
        :type user_id: str
        :param user_factor_id: target factor id
        :type user_factor_id: str
        :param factor_enroll_request: data to update the factor
        :type factor_enroll_request: FactorEnrollRequest
        :rtype: Factor
        """
        response = ApiClient.put_path(self, '/{0}/factors/{1}'.format(user_id, user_factor_id), factor_enroll_request)
        return Utils.deserialize(response.text, Factor)

    def reset_factor(self, user_id, user_factor_id):
        """Reset an enrolled factor

        :param user_id: target user id
        :type user_id: str
        :param user_factor_id: target factor id
        :type user_factor_id: str
        :rtype: None
        """
        ApiClient.delete_path(self, '/{0}/factors/{1}'.format(user_id, user_factor_id))

    # FACTOR LIFECYCLE

    def activate_factor(self, user_id, user_factor_id, passcode, next_passcode=None):
        """Activate an enrolled factor

        :param user_id: target user id
        :type user_id: str
        :param user_factor_id: target factor id
        :type user_factor_id: str
        :param passcode: code required for activation
        :type passcode: str
        :param next_passcode: code usually required for TOTP
        :type next_passcode: str
        :rtype: Factor
        """
        request = {
            'passCode': passcode,
            'next_passcode': next_passcode
        }
        response = ApiClient.post_path(self, '/{0}/factors/{1}/lifecycle/activate'.format(user_id, user_factor_id), request)
        return Utils.deserialize(response.text, Factor)

    def resend_code(self, user_id, user_factor_id):
        """Resend code for a factor

        :param user_id: target user id
        :type user_id: str
        :param user_factor_id: target factor id
        :type user_factor_id: str
        :return:
        """
        response = ApiClient.post_path(self, '/{0}/factors/{1}/resend'.format(user_id, user_factor_id))
        return Utils.deserialize(response.text, Factor)

    def verify_factor(self, user_id, user_factor_id, activation_token=None, answer=None, passcode=None, next_passcode=None):
        """Verify an enrolled factor

        :param user_id: target user id
        :type user_id: str
        :param user_factor_id: target factor id
        :type user_factor_id: str
        :param activation_token: token required for activation
        :type activation_token: str
        :param answer: answer usually required for a question factor
        :type answer: str
        :param passcode: code required for verification
        :type passcode: str
        :param next_passcode: code usually required for TOTP
        :type next_passcode: str
        :return:
        """
        request = {
            'activationToken': activation_token,
            'answer': answer,
            'passCode': passcode,
            'nextPassCode': next_passcode
        }
        response = ApiClient.post_path(self, '/{0}/factors/{1}/verify'.format(user_id, user_factor_id), request)
        return Utils.deserialize(response.text, FactorVerificationResponse)

    # FACTOR DEVICE CRUD

    def enroll_factor_device(self, user_id, factor_enroll_request):
        """Enroll a factor device for a user

        :param user_id: target user id
        :type user_id: str
        :param factor_enroll_request: data to enroll the factor device
        :type factor_enroll_request: FactorEnrollRequest
        :rtype: FactorDevice
        """
        response = ApiClient.post_path(self, '/{0}/devices'.format(user_id), factor_enroll_request)
        return Utils.deserialize(response.text, FactorDevice)

    def get_factor_device(self, user_id, user_factor_id, device_id):
        """Get a factor device for a user

        :param user_id: target user id
        :type user_id: str
        :param user_factor_id: target factor id
        :type user_factor_id: str
        :param device_id: target factor device id
        :type device_id: str
        :rtype: FactorDevice
        """
        response = ApiClient.get_path(self, '/{0}/factors/{1}/device/{2}'.format(user_id, user_factor_id, device_id))
        return Utils.deserialize(response.text, FactorDevice)

    def update_factor_device(self, user_id, factor_device_request):
        """Update a factor device for a user

        :param user_id: target user id
        :type user_id: str
        :param factor_device_request: data to update the factor device
        :type factor_device_request: FactorDeviceRequest
        :rtype: FactorDevice
        """
        response = ApiClient.post_path(self, '/{0}/factors/{1}'.format(user_id), factor_device_request)
        return Utils.deserialize(response.text, FactorDevice)

    # FACTOR DEVICE LIFECYCLE

    def activate_factor_device(self, user_id, user_factor_id, device_id, passcode):
        """Activate a factor device for a user

        :param user_id: target user id
        :type user_id: str
        :param user_factor_id: target factor id
        :type user_factor_id: str
        :param device_id: target factor device id
        :type device_id: str
        :param passcode: code required to activate the factor device
        :type passcode: str
        :rtype: FactorDevice
        """
        request = {
            'passCode': passcode
        }
        response = ApiClient.post_path(self, '/{0}/factors/{1}/devices/{2}/lifecycle/activate'.format(
            user_id, user_factor_id, device_id), request)
        return Utils.deserialize(response.text, Factor)


from okta.framework.ApiClient import ApiClient
from okta.framework.Utils import Utils
from okta.framework.PagedResults import PagedResults
from okta.models.app.AppInstance import AppInstance


class AppInstanceClient(ApiClient):
    def __init__(self, base_url, api_token):
        ApiClient.__init__(self, base_url + '/api/v1/apps', api_token)

    # CRUD

    def get_app_instances(self, limit=None, filter_string=None):
        """Get a list of AppInstances

        :param limit: maximum number of apps to return
        :type limit: int or None
        :param filter_string: string to filter users
        :type filter_string: str or None
        :rtype: list of AppInstance
        """
        params = {
            'limit': limit,
            'filter': filter_string
        }
        response = ApiClient.get_path(self, '/', params=params)
        return Utils.deserialize(response.text, AppInstance)

    def get_paged_app_instances(self, limit=None, filter_string=None, after=None, url=None):
        """Get a paged list of AppInstances

        :param limit: maximum number of apps to return
        :type limit: int or None
        :param filter_string: string to filter apps
        :type filter_string: str or None
        :param after: app id that filtering will resume after
        :type after: str
        :param url: url that returns a list of AppInstance
        :type url: str
        :rtype: PagedResults of AppInstance
        """
        if url:
            response = ApiClient.get(self, url)

        else:
            params = {
                'limit': limit,
                'after': after,
                'filter': filter_string
            }
            response = ApiClient.get_path(self, '/', params=params)

        return PagedResults(response, AppInstance)

    def create_app_instance(self, app_instance):
        """Create a app instance

        :param app_instance: the data to create a user
        :type app_instance: AppInstance
        :rtype: AppInstance
        """
        response = ApiClient.post_path(self, '/', app_instance)
        return Utils.deserialize(response.text, AppInstance)

    def get_app_instance(self, id):
        """Get a single app

        :param id: the app id
        :type id: str
        :rtype: AppInstance
        """
        response = ApiClient.get_path(self, '/{0}'.format(id))
        return Utils.deserialize(response.text, AppInstance)

    def update_app_instance(self, app_instance):
        """Update an app

        :param app_instance: the app to update
        :type app_instance: AppInstance
        :rtype: AppInstance
        """
        return self.update_app_instance_by_id(app_instance.id, app_instance)

    def update_app_instance_by_id(self, id, app_instance):
        """Update an app, defined by an id

        :param id: the target app id
        :type id: str
        :param app_instance: the data to update the target app
        :type app_instance: AppInstance
        :rtype: AppInstance
        """
        response = ApiClient.put_path(self, '/{0}'.format(id), app_instance)
        return Utils.deserialize(response.text, AppInstance)

    def delete_app_instance(self, id):
        """Delete app by target id

        :param id: the target app id
        :type id: str
        :return: None
        """
        ApiClient.delete_path(self, '/{0}'.format(id))

    # LIFECYCLE

    def activate_app_instance(self, id):
        """Activate app by target id

        :param id: the target app id
        :type id: str
        :return: None
        """
        ApiClient.post_path(self, '/{0}/lifecycle/activate'.format(id), None)

    def deactivate_app_instance(self, id):
        """Deactivate app by target id

        :param id: the target app id
        :type id: str
        :return: None
        """
        ApiClient.post_path(self, '/{0}/lifecycle/deactivate'.format(id), None)


class Target:

    types = {
        'id': str,
        'displayName': str,
        'objectType': str
    }

    def __init__(self):

        # Unique key for actor
        self.id = None  # str

        # Name of actor used for display purposes
        self.displayName = None  # str

        # User, Client, or AppInstance
        self.objectType = None  # str



from okta.models.user import Password


class ChangePasswordRequest:

    types = {
        'oldPassword': Password,
        'newPassword': Password
    }

    def __init__(self):

        self.oldPassword = None  # Password

        self.newPassword = None  # Password



from datetime import datetime
from okta.models.factor.FactorProfile import FactorProfile


class Factor:

    types = {
        'id': str,
        'factorType': str,
        'provider': str,
        'status': str,
        'created': datetime,
        'lastUpdated': datetime,
        'profile': FactorProfile
    }

    def __init__(self):

        # unique key for factor
        self.id = None  # str

        # type of factor
        self.factorType = None  # str

        # factor provider
        self.provider = None  # str

        # status of factor
        self.status = None  # str

        # timestamp when factor was created
        self.created = None  # datetime

        # timestamp when factor was last updated
        self.lastUpdated = None  # datetime

        # profile of a supported factor
        self.profile = None  # FactorProfile

        self.links = None  # Map<String, LinksUnion>

        self.embedded = None  # Map<String, Object>


class OktaError(Exception):

    def __init__(self, error):
        if error is None:
            error = {}

        Exception.__init__(self, error.get('errorSummary'))

        self.error_causes = error.get('errorCauses')
        self.error_code = error.get('errorCode')
        self.error_id = error.get('errorId')
        self.error_link = error.get('errorLink')
        self.error_summary = error.get('errorSummary')


from okta.framework.ApiClient import ApiClient
from okta.framework.Utils import Utils
from okta.models.session.Credentials import Credentials
from okta.models.session.Session import Session


class SessionsClient(ApiClient):
    def __init__(self, base_url, api_token):
        ApiClient.__init__(self, base_url + '/api/v1/sessions', api_token)

    # CRUD

    def create_session(self, username, password, additional_fields=None):
        """Create a session

        :param username: the user's username
        :type username: str
        :param password: the user's password
        :type password: str
        :param additional_fields: additional fields that will be included in the response
        :type additional_fields: str
        :rtype: Session
        """
        creds = Credentials()
        creds.username = username
        creds.password = password
        params = {'additionalFields': additional_fields}
        response = ApiClient.post_path(self, '/', creds, params=params)
        return Utils.deserialize(response.text, Session)

    def create_session_with_cookie_token(self, username, password):
        """Create a session that contains a cookie token

        :param username: the user's username
        :type username: str
        :param password: the user's password
        :type password: str
        :rtype: Session
        """
        return self.create_session(username, password, 'cookieToken')

    def create_session_with_cookie_token_url(self, username, password):
        """Create a session that contains a cookie token url

        :param username: the user's username
        :type username: str
        :param password: the user's password
        :type password: str
        :rtype: Session
        """
        return self.create_session(username, password, 'cookieTokenUrl')

    def create_session_by_session_token(self, session_token, additional_fields=None):
        """Create a session using a session token

        :param session_token: a token that can be exchanged for a session
        :type session_token: str
        :param additional_fields: additional fields that will be included in the response
        :type additional_fields: str
        :rtype: Session
        """
        data = {'sessionToken': session_token}
        params = {'additionalFields': additional_fields}
        response = ApiClient.post_path(self, '/', data, params=params)
        return Utils.deserialize(response.text, Session)

    def validate_session(self, id):
        """Validate a session

        :param id: the target session id
        :rtype: Session
        """
        response = ApiClient.get_path(self, '/{0}'.format(id))
        return Utils.deserialize(response.text, Session)

    def extend_session(self, id):
        """Extend a session's lifespan

        :param id: the target session id
        :rtype: Session
        """
        response = ApiClient.put_path(self, '/{0}'.format(id), None)
        return Utils.deserialize(response.text, Session)

    def clear_session(self, id):
        """Terminate a session

        :param id: the target session id
        :rtype: Session
        """
        ApiClient.delete_path(self, '/{0}'.format(id))


from okta.framework.ApiClient import ApiClient
from okta.framework.Utils import Utils
from okta.models.usergroup.UserGroup import UserGroup
from okta.framework.PagedResults import PagedResults


class UserGroupsClient(ApiClient):
    def __init__(self, base_url, api_token):
        ApiClient.__init__(self, base_url + '/api/v1/groups', api_token)

    # CRUD

    def get_groups(self, limit=None, query=None):
        """Get a list of UserGroups

        :param limit: maximum number of groups to return
        :type limit: int or None
        :param query: string to search group names
        :type query: str or None
        :rtype: list of UserGroup
        """
        params = {
            'limit': limit,
            'q': query
        }
        response = ApiClient.get_path(self, '/', params=params)
        return Utils.deserialize(response.text, UserGroup)

    def get_paged_groups(self, limit=None, after=None, url=None):
        """Get a paged list of UserGroups

        :param limit: maximum number of groups to return
        :type limit: int or None
        :param after: group id that filtering will resume after
        :type after: str
        :param url: url that returns a list of UserGroup
        :type url: str
        :rtype: PagedResults of UserGroup
        """
        if url:
            response = ApiClient.get(self, url)

        else:
            params = {
                'limit': limit,
                'after': after
            }
            response = ApiClient.get_path(self, '/', params=params)

        return PagedResults(response, User)

    def get_group(self, gid):
        """Get a single group

        :param gid: the group id
        :type gid: str
        :rtype: UserGroup
        """
        response = ApiClient.get_path(self, '/{0}'.format(gid))
        return Utils.deserialize(response.text, UserGroup)

    def update_group(self, group):
        """Update a group

        :param group: the group to update
        :type group: UserGroup
        :rtype: UserGroup
        """
        return self.update_group_by_id(group.id, group)

    def update_group_by_id(self, gid, group):
        """Update a group, defined by an id

        :param gid: the target group id
        :type gid: str
        :param group: the data to update the target group
        :type group: UserGroup
        :rtype: UserGroup
        """
        response = ApiClient.put_path(self, '/{0}'.format(gid), group)
        return Utils.deserialize(response.text, UserGroup)

    def create_group(self, group):
        """Create a group

        :param group: the data to create a group
        :type group: UserGroup
        :rtype: UserGroup
        """
        response = ApiClient.post_path(self, '/', group)
        return Utils.deserialize(response.text, UserGroup)

    def delete_group(self, gid):
        """Delete group by target id

        :param gid: the target group id
        :type gid: str
        :return: None
        """
        response = ApiClient.delete_path(self, '/{0}'.format(gid))
        return Utils.deserialize(response.text, UserGroup)

    def add_user_to_group(self, group, user):
        """Add a user to a group

        :param group: the target group
        :type group: UserGroup
        :param user: the target user
        :type user: User
        :return: None
        """
        return self.add_user_to_group_by_id(group.id, user.id)

    def add_user_to_group_by_id(self, gid, uid):
        """Add a user to a group

        :param gid: the target group id
        :type gid: str
        :param uid: the target user id
        :type uid: str
        :return: None
        """
        response = ApiClient.put_path(self, '/{0}/users/{1}'.format(gid, uid))


from okta.models.Embedded import Embedded


class AuthResult:

    types = {
        'stateToken': str,
        'status': str,
        'expiresAt': str,
        'relayState': str,
        'factorResult': str,
        'factorResultMessage': str,
        'recoveryToken': str,
        'sessionToken': str,
        'idToken': str,
        '_embedded': Embedded
    }

    alt_names = {
        '_embedded': 'embedded'
    }

    def __init__(self):

        self.stateToken = None  # str

        self.status = None  # str

        self.expiresAt = None  # str

        self.relayState = None  # str

        self.factorResult = None  # str

        self.factorResultMessage = None  # str

        self.recoveryToken = None  # str

        self.sessionToken = None  # str

        self.idToken = None  # str

        self.embedded = None # Embedded


class FactorProfile:

    types = {
        'question': str,
        'questionText': str,
        'answer': str,
        'phoneNumber': str,
        'credentialId': str
    }

    def __init__(self):

        # unique key for question
        self.question = None  # str

        # display text for question
        self.questionText = None  # str

        # answer to question
        self.answer = None  # str

        # phone number of mobile device
        self.phoneNumber = None  # str

        # unique id for instance
        self.credentialId = None  # str



import requests
import json
import time
from okta.framework.Serializer import Serializer
from okta.framework.OktaError import OktaError
import six


class ApiClient(object):

    def __init__(self, base_url, api_token):
        self.base_url = base_url
        self.api_token = api_token
        self.api_version = 1
        self.max_attempts = 4

        if not self.base_url:
            raise ValueError('Invalid base_url')

        if not self.api_token:
            raise ValueError('Invalid api_token')

        self.headers = {
            'Accept': 'application/json',
            'Content-Type': 'application/json',
            'Authorization': 'SSWS ' + api_token
        }

    def get(self, url, params=None, attempts=0):
        params_str = self.__dict_to_query_params(params)
        resp = requests.get(url + params_str, headers=self.headers)
        attempts += 1
        if self.__check_response(resp, attempts):
            return resp
        else:
            return self.get(url, params, attempts)

    def put(self, url, data=None, params=None, attempts=0):
        d = json.dumps(data, cls=Serializer)
        params_str = self.__dict_to_query_params(params)
        resp = requests.put(url + params_str, data=d, headers=self.headers)
        attempts += 1
        if self.__check_response(resp, attempts):
            return resp
        else:
            return self.put(url, data, params, attempts)

    def post(self, url, data=None, params=None, attempts=0):
        d = json.dumps(data, cls=Serializer)
        params_str = self.__dict_to_query_params(params)
        resp = requests.post(url + params_str, data=d, headers=self.headers)
        attempts += 1
        if self.__check_response(resp, attempts):
            return resp
        else:
            return self.post(url, data, params, attempts)

    def delete(self, url, params=None, attempts=0):
        params_str = self.__dict_to_query_params(params)
        resp = requests.delete(url + params_str, headers=self.headers)
        attempts += 1
        if self.__check_response(resp, attempts):
            return resp
        else:
            return self.delete(url, params, attempts)

    def get_path(self, url_path, params=None):
        return self.get(self.base_url + url_path, params)

    def put_path(self, url_path, data=None, params=None):
        return self.put(self.base_url + url_path, data, params)

    def post_path(self, url_path, data=None, params=None):
        return self.post(self.base_url + url_path, data, params)

    def delete_path(self, url_path, params=None):
        return self.delete(self.base_url + url_path, params)

    def __check_response(self, resp, attempts=1):
        if resp is None:
            raise ValueError("A response wasn't received")

        if 200 <= resp.status_code < 300:
            return True

        # If we made it this far, we need to handle an exception
        if attempts >= self.max_attempts or resp.status_code != 429:
            raise OktaError(json.loads(resp.text))

        # Assume we're going to retry with exponential backoff
        time.sleep(2 ** (attempts - 1))

        return False

    @staticmethod
    def __dict_to_query_params(d):
        if d is None or len(d) == 0:
            return ''

        param_list = [param + '=' + (str(value).lower() if type(value) == bool else str(value))
                      for param, value in six.iteritems(d) if value is not None]
        return '?' + "&".join(param_list)


class Link:

    types = {
        'href': str,
        'method': str,
        'type': str,
        'name': str,
        'hints': dict
    }

    def __init__(self):

        self.href = None

        self.method = None

        self.type = None

        self.name = None

        self.hints = None


from okta.models.user.User import User
from okta.models.factor.Factor import Factor


class Embedded:

    types = {
        'user': User,
        'factors': Factor
    }

    def __init__(self):

        self.user = None

        self.factors = None


from okta.framework.ApiClient import ApiClient
from okta.framework.Utils import Utils
from okta.models.event.Event import Event
from okta.framework.PagedResults import PagedResults


class EventsClient(ApiClient):
    def __init__(self, base_url, api_token):
        ApiClient.__init__(self, base_url + '/api/v1/events', api_token)

    def get_events(self, limit=None, start_date=None, filter_string=None):
        """Get a list of Events

        :param limit: maximum number of events to return
        :type limit: int or None
        :param filter_string: string to filter events
        :type filter_string: str or None
        :rtype: list of Event
        """
        params = {
            'limit': limit,
            'startDate': start_date,
            'filter': filter_string
        }
        response = ApiClient.get_path(self, '/', params=params)

        return Utils.deserialize(response.text, Event)

    def get_paged_events(self, limit=None, start_date=None, after=None, filter_string=None, url=None):
        """Get a paged list of Events

        :param limit: maximum number of events to return
        :type limit: int or None
        :param filter_string: string to filter events
        :type filter_string: str or None
        :param after: event id that filtering will resume after
        :type after: str or None
        :param url: url that returns a list of Event
        :type url: str or None
        :rtype: PagedResults of Event
        """
        if url:
            response = ApiClient.get(self, url)

        else:
            params = {
                'limit': limit,
                'startDate': start_date,
                'after': after,
                'filter': filter_string
            }
            response = ApiClient.get_path(self, '/', params=params)

        return PagedResults(response, Event)


from datetime import datetime
from okta.models.app.AppUserCredentials import AppUserCredentials
from okta.models.app.AppUserProfile import AppUserProfile


class AppUser:

    types = {
        'id': str,
        'externalId': str,
        'created': datetime,
        'lastUpdated': datetime,
        'scope': str,
        'status': str,
        'statusChanged': datetime,
        'passwordChanged': datetime,
        'syncState': str,
        'lastSynced': datetime,
        'credentials': AppUserCredentials,
        'profile': AppUserProfile
    }

    def __init__(self):

        # unique key of user
        self.id = None  # str

        # id of user in target app (must be imported or provisioned)
        self.externalId = None  # str

        # timestamp when user was created
        self.created = None  # datetime

        # timestamp when user was last updated
        self.lastUpdated = None  # datetime

        # toggles the assignment between user or group scope
        self.scope = None  # str

        # status of app user
        self.status = None  # str

        # timestamp when status last changed
        self.statusChanged = None  # datetime

        # timestamp when app password last changed
        self.passwordChanged = None  # datetime

        # synchronization state for app user
        self.syncState = None  # str

        # timestamp when last sync operation was executed
        self.lastSynced = None  # datetime

        # credentials for assigned app
        self.credentials = None  # AppUserCredentials

        # app-specific profile for the user
        self.profile = None  # AppUserProfile

        self.links = None  # Map<String, LinksUnion>

        self.embedded = None  # Map<String, Object>



class AppUserCredentials:

    types = {
        'userName': str,
        'password': str
    }

    def __init__(self):

        # username for app
        self.userName = None  # str

        # password for app
        self.password = None  # str



class FactorRequest:

    types = {
        'answer': str,
        'passCode': str,
        'nextPassCode': str
    }

    def __init__(self):

        self.answer = None  # str

        self.passCode = None  # str

        self.nextPassCode = None  # str



class Accessibility:

    types = {
        'selfService': bool,
        'errorRedirectUrl': str
    }

    def __init__(self):

        # Enable self-service application assignment
        self.selfService = None  # bool

        # Custom error page for this application
        self.errorRedirectUrl = None  # str



from Verification import Verification


class FactorEnrollRequest:

    types = {
        'factorType': str,
        'provider': str,
        'verify': Verification,
        'profile': dict
    }

    def __init__(self):

        self.factorType = None  # str

        self.provider = None  # str

        self.verify = None  # Verification

        self.profile = None  # Map<String, Object>



from okta.models.user.Provider import Provider
from okta.models.user.Password import Password
from okta.models.user.RecoveryQuestion import RecoveryQuestion


class LoginCredentials:

    types = {
        'password': Password,
        'recovery_question': RecoveryQuestion,
        'provider': Provider
    }

    def __init__(self):

        self.password = None  # Password

        self.recovery_question = None  # RecoveryQuestion

        self.provider = None  # Provider



class Session:

    types = {
        'id': str,
        'userId': str,
        'mfaActive': bool,
        'cookieToken': str,
        'cookieTokenUrl': str
    }

    def __init__(self):

        # unique key for session
        self.id = None  # str

        # unique key for the user
        self.userId = None  # str

        # indicates whether the user has enrolled a valid MFA credential
        self.mfaActive = None  # bool

        # One-time token which can be used to obtain a session cookie for your organization
        # by visiting either an applications embed link or a session redirect URL.
        self.cookieToken = None  # str

        # URL for a a transparent 1x1 pixel image which contains a one-time token which
        # when visited sets the session cookie in your browser for your organization.
        self.cookieTokenUrl = None  # str



from json import dumps, JSONEncoder
from datetime import datetime
from okta.framework.Utils import Utils


class Serializer(JSONEncoder):
    def default(self, obj):

        if isinstance(obj, datetime):
            return obj.strftime('dt(%Y-%m-%dT%H:%M:%SZ)')
        elif isinstance(obj, object):
            return Utils.remove_nulls(obj.__dict__)
        else:
            return JSONEncoder.default(self, obj)


from okta.models.usergroup.UserGroupProfile import UserGroupProfile
from okta.models.Link import Link


class UserGroup:

    types = {
        'id': str,
        'objectClass': str,
        'profile': UserGroupProfile
    }

    dict_types = {
        '_links': Link
    }

    alt_names = {
        '_links': 'links'
    }

    def __init__(self, **kwargs):

        # unique key for group
        self.id = None  # str

        # determines the groups profile
        self.objectClass = None  # enum

        # the groups profile attributes
        self.profile = None  # UserGroupProfile

        self.links = None

        # Populate profile
        profile_attrs = ['name', 'description']
        for attr in profile_attrs:
            if attr in kwargs:
                self.profile = self.profile or UserGroupProfile()
                setattr(self.profile, attr, kwargs[attr])


class UserProfile:

    types = {
        'login': str,
        'email': str,
        'secondEmail': str,
        'firstName': str,
        'lastName': str,
        'mobilePhone': str
    }

    def __init__(self):

        self.login = None  # str

        self.email = None  # str

        self.secondEmail = None  # str

        self.firstName = None  # str

        self.lastName = None  # str

        self.mobilePhone = None  # str



from okta.framework.ApiClient import ApiClient
from okta.framework.Utils import Utils
from okta.framework.PagedResults import PagedResults
from okta.models.user.User import User
from okta.models.user.TempPassword import TempPassword
from okta.models.user.ResetPasswordToken import ResetPasswordToken
from okta.models.user.LoginCredentials import LoginCredentials


class UsersClient(ApiClient):
    def __init__(self, base_url, api_token):
        ApiClient.__init__(self, base_url + '/api/v1/users', api_token)

    # CRUD

    def get_users(self, limit=None, query=None, filter_string=None):
        """Get a list of Users

        :param limit: maximum number of users to return
        :type limit: int or None
        :param query: string to search users' first names, last names, and emails
        :type query: str or None
        :param filter_string: string to filter users
        :type filter_string: str or None
        :rtype: list of User
        """
        params = {
            'limit': limit,
            'q': query,
            'filter': filter_string
        }
        response = ApiClient.get_path(self, '/', params=params)
        return Utils.deserialize(response.text, User)

    def get_user(self, uid):
        """Get a single user

        :param uid: the user id or login
        :type uid: str
        :rtype: User
        """
        response = ApiClient.get_path(self, '/{0}'.format(uid))
        return Utils.deserialize(response.text, User)

    def update_user(self, user):
        """Update a user

        :param user: the user to update
        :type user: User
        :rtype: User
        """
        return self.update_user_by_id(user.id, user)

    def update_user_by_id(self, uid, user):
        """Update a user, defined by an id

        :param uid: the target user id
        :type uid: str
        :param user: the data to update the target user
        :type user: User
        :rtype: User
        """
        response = ApiClient.put_path(self, '/{0}'.format(uid), user)
        return Utils.deserialize(response.text, User)

    def create_user(self, user, activate=False):
        """Create a user

        :param user: the data to create a user
        :type user: User
        :param activate: whether to activate the user
        :type activate: bool
        :rtype: User
        """
        if activate is None:
            response = ApiClient.post_path(self, '/', user)
        else:
            params = {
                'activate': activate
            }
            response = ApiClient.post_path(self, '/', user, params=params)
        return Utils.deserialize(response.text, User)

    def delete_user(self, uid):
        """Delete user by target id

        :param uid: the target user id
        :type uid: str
        :return: None
        """
        response = ApiClient.delete_path(self, '/{0}'.format(uid))
        return Utils.deserialize(response.text, User)

    def get_paged_users(self, limit=None, filter_string=None, after=None, url=None):
        """Get a paged list of Users

        :param limit: maximum number of users to return
        :type limit: int or None
        :param filter_string: string to filter users
        :type filter_string: str or None
        :param after: user id that filtering will resume after
        :type after: str
        :param url: url that returns a list of User
        :type url: str
        :rtype: PagedResults of User
        """
        if url:
            response = ApiClient.get(self, url)

        else:
            params = {
                'limit': limit,
                'after': after,
                'filter': filter_string
            }
            response = ApiClient.get_path(self, '/', params=params)

        return PagedResults(response, User)

    # LIFECYCLE

    def activate_user(self, uid):
        """Activate user by target id

        :param uid: the target user id
        :type uid: str
        :return: User
        """
        response = ApiClient.post_path(self, '/{0}/lifecycle/activate'.format(uid))
        return Utils.deserialize(response.text, User)

    def deactivate_user(self, uid):
        """Deactivate user by target id

        :param uid: the target user id
        :type uid: str
        :return: User
        """
        response = ApiClient.post_path(self, '/{0}/lifecycle/deactivate'.format(uid))
        return Utils.deserialize(response.text, User)

    def unlock_user(self, uid):
        """Unlock user by target id

        :param uid: the target user id
        :type uid: str
        :return: User
        """
        response = ApiClient.post_path(self, '/{0}/lifecycle/unlock'.format(uid))
        return Utils.deserialize(response.text, User)

    def reset_password(self, uid, send_email=True):
        """Reset user's password by target user id

        :param uid: the target user id
        :type uid: str
        :param send_email: whether a password reset email should be sent
        :type send_email: bool
        :return: None or ResetPasswordToken
        """
        params = {
            'sendEmail': send_email
        }
        response = ApiClient.post_path(self, '/{0}/lifecycle/reset_password'.format(uid), params=params)
        return Utils.deserialize(response.text, ResetPasswordToken)

    def change_password(self, uid, old_password, new_password):
        """Change user's password by target user id

        :param uid: the target user id
        :type uid: str
        :param old_password: the user's old password
        :type old_password: str
        :param new_password: the desired new password
        :type new_password: str
        :return: None or LoginCredentials
        """
        data = {
            'oldPassword': {
                'value': old_password
            },
            'newPassword': {
                'value': new_password
            }
        }
        response = ApiClient.post_path(self, '/{0}/credentials/change_password'.format(uid), data)
        return Utils.deserialize(response.text, LoginCredentials)

    def expire_password(self, uid, temp_password=False):
        """Expire user's password by target user id

        :param uid: the target user id
        :type uid: str
        :param temp_password: whether a temporary password should be set
        :type temp_password: bool
        :return: None or TempPassword
        """
        if not temp_password:
            ApiClient.post_path(self, '/{0}/lifecycle/expire_password'.format(uid))
        else:
            params = {
                'tempPassword': temp_password
            }
            response = ApiClient.post_path(self, '/{0}/lifecycle/expire_password'.format(uid), params=params)
            return Utils.deserialize(response.text, TempPassword)



from okta.models.app.Hide import Hide
from okta.models.app.AppLinks import AppLinks


class Visibility:

    types = {
        'autoSubmitToolbar': bool,
        'hide': Hide,
        'appLinks': AppLinks
    }

    def __init__(self):

        # Automatically log in when user lands on login page
        self.autoSubmitToolbar = None  # bool

        # Hides this app for specific end-user apps
        self.hide = None  # Hide

        # Displays specific appLinks for the app
        self.appLinks = None  # AppLinks



from datetime import datetime


class AppGroup:

    types = {
        'id': str,
        'lastUpdated': datetime,
        'priority': int
    }

    def __init__(self):

        # unique key of group
        self.id = None  # str

        # timestamp when app group was last updated
        self.lastUpdated = None  # datetime

        # priority of group assignment
        self.priority = None  # int



class FactorVerificationResponse:

    types = {
        'factorResult': str,
        'state': str,
        'factorResultMessage': str,
        'expiryTime': str
    }

    def __init__(self):

        self.factorResult = None  # str

        self.state = None  # str

        self.factorResultMessage = None  # str

        self.expiryTime = None  # str

        self.links = None  # Map<String, LinksUnion>

        self.embedded = None  # Map<String, Object>



from setuptools import setup, find_packages
from codecs import open
from os import path
import ast
import re

# Find the version in the package's __init__.py
_version_re = re.compile(r'__version__\s+=\s+(.*)')

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'okta', '__init__.py'), 'rb') as f:
    version = str(ast.literal_eval(_version_re.search(
        f.read().decode('utf-8')).group(1)))

# Get the long description
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='okta3',
    version=version,
    description='Okta client APIs',
    long_description=long_description,
    license='Apache License 2.0',
    author='Okta',
    author_email='developers@okta.com',
    url='https://github.com/okta/oktasdk-python',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'License :: OSI Approved :: Apache Software License'],
    packages=find_packages(),
    install_requires=[
        'requests>=2.5.3',
        'python-dateutil>=2.4.2',
        'six>=1.9.0'
    ]
)



class UserGroupProfile:

    types = {
        'name': str,
        'description': str,
        'samAccountName': str,
        'dn': str,
        'windowsDomainQualifiedName': str,
        'externalId': str
    }

    def __init__(self):

        # name of the group
        self.name = None  # str

        # description of the group
        self.description = None  # str

        # pre-windows 2000 name of the windows group
        self.samAccountName = None  # str

        # the distinguished name of the windows group
        self.dn = None  # str

        # fully-qualified name of the windows group
        self.windowsDomainQualifiedName = None  # str

        # base-64 encoded GUID (objectGUID) of the windows group
        self.externalId = None  # str



from .ActivationResponse import ActivationResponse
from .ChangePasswordRequest import ChangePasswordRequest
from .ChangeRecoveryQuestionRequest import ChangeRecoveryQuestionRequest
from .LoginCredentials import LoginCredentials
from .Password import Password
from .Provider import Provider
from .RecoveryQuestion import RecoveryQuestion
from .ResetPasswordToken import ResetPasswordToken
from .TempPassword import TempPassword
from .User import User
from .UserProfile import UserProfile



class AppSettings:

    types = {
        'url': str,
        'requestIntegration': bool,
        'authURL': str,
        'usernameField': str,
        'passwordField': str,
        'buttonField': str,
        'extraFieldSelector': str,
        'extraFieldValue': str,
        'optionalField1': str,
        'optionalField1Value': str,
        'optionalField2': str,
        'optionalField2Value': str,
        'optionalField3': str,
        'optionalField3Value': str
    }

    def __init__(self):

        # The URL of the login page for this app
        self.url = None  # str

        # Would you like Okta to add an integration for this app?
        self.requestIntegration = None  # bool

        # The URL of the authenticating site for this app
        self.authURL = None  # str

        # CSS selector for the username field in the login form
        self.usernameField = None  # str

        # CSS selector for the password field in the login form
        self.passwordField = None  # str

        # CSS selector for the login button in the login form
        self.buttonField = None  # str

        # CSS selector for the extra field in the form
        self.extraFieldSelector = None  # str

        # Value for extra field form field
        self.extraFieldValue = None  # str

        # Name of the optional parameter in the login form
        self.optionalField1 = None  # str

        # Name of the optional value in the login form
        self.optionalField1Value = None  # str

        # Name of the optional parameter in the login form
        self.optionalField2 = None  # str

        # Name of the optional value in the login form
        self.optionalField2Value = None  # str

        # Name of the optional parameter in the login form
        self.optionalField3 = None  # str

        # Name of the optional value in the login form
        self.optionalField3Value = None  # str
